<?php

declare(strict_types=1);

namespace Drupal\field_file_and_translations;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file\FileInterface;

/**
 * Handles the translation of Media Documents in bulk.
 */
class ContentMultiUploadTranslationHandler {
  use StringTranslationTrait;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The configuration factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new Multiupload Translation Handler.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(LanguageManagerInterface $languageManager, ConfigFactoryInterface $configFactory) {
    $this->languageManager = $languageManager;
    $this->configFactory = $configFactory;
  }

  /**
   * Helper function to include the entity translations on insert / update.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity instance.
   */
  public function includeTranslations(EntityInterface $entity): void {
    $defaultLanguage = \Drupal::languageManager()->getDefaultLanguage()->getId();
    $fields_with_file_translations = $this->getFileTranslations($entity);

    foreach ($fields_with_file_translations as $fieldName => $file_translations) {
      if (isset($file_translations[$defaultLanguage])) {
        $entity->set($fieldName, $file_translations[$defaultLanguage]->id());
        unset($file_translations[$defaultLanguage]);
      }
        // Iterate over the current file translations.
        $this->createOrEditTranslations($file_translations, $entity, $fieldName);
    }


  }

  /**
   * Custom helper to get the file translations.
   */
  private function getFileTranslations($entity) {
    // Get the referenced entities from the custom field.
    $file_translations = [];
    $fields = $this->getFileAndTranslationFields($entity);
    foreach ($fields as $field) {
      foreach ($field->referencedEntities() as $file) {
        $filename = mb_strtolower($file->getFilename($file));
        $language_suffix = $this->getLanguageSuffix($filename);
        $languageCodes = $this->getLanguageCodes();
        if (in_array($language_suffix, $languageCodes, true)) {
          $langcode = array_search($language_suffix, $languageCodes, true);
          $file_translations[$field->getName()][$langcode] = $file;
        }
      }
    }

    return $file_translations;
  }

  /**
   * Get the language suffix from the name of the given file.
   */
  private function getLanguageSuffix(string $filename): string {
    if (preg_match('/[_-]([a-zA-Z]{2})(_\d+)?\.[a-zA-Z]+$/i', str_replace(' ', '', $filename), $match)) {
      return mb_strtolower($match[1]);
    }
    return $this->languageManager->getDefaultLanguage()->getId();
  }

  /**
   * Create media translation entities.
   */
  private function createOrEditTranslations(array $file_translations, EntityInterface $entity, $fieldName): void {
    foreach ($file_translations as $langCode => $file) {
      $entity_reference = $entity->hasTranslation($langCode) ? $entity->getTranslation($langCode) : $entity->addTranslation($langCode);

      $file->set('langcode', $langCode);
      $file->save();

      // Update the common translatable fields.
      $entity_reference->set($fieldName, $file->id());
      $entity_reference->set('status', $entity->get('status')->value);

      // Set display and description to avoid empty.
      $entity_reference->{$fieldName}->description = !empty($entity_reference->oe_media_file->description) ?: '';
      $entity_reference->{$fieldName}->display = !empty($entity_reference->oe_media_file->display) ?: 1;
      $entity_reference->{$fieldName}->filename = $file->getFilename();

    }
  }

  /**
   * Get all 'file_and_translations' fields from an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   */
  function getFileAndTranslationFields(EntityInterface $entity) {
    $file_and_translations_fields = [];
    foreach ($entity as $field_name => $field) {
     if ($field->getFieldDefinition()->getType() === 'file_and_translations') {
        $file_and_translations_fields[$field_name] = $field;
      }
    }

    return $file_and_translations_fields;
  }

  /**
   * Checks for valid language suffix in file name.
   *
   * @param \Drupal\file\FileInterface $file
   *   A file entity.
   */
  public function fileValidateLanguageSuffix(FileInterface $file) {
    $errors = [];
    $filename = $file->getFilename();

    $current_language = $this->languageManager->getCurrentLanguage()->getId();
    $default_language = $this->languageManager->getDefaultLanguage()->getId();
    $file_language_suffix = $this->getLanguageSuffix($filename);

    $enabled_langcodes = $this->getLanguageCodes();

    if (!in_array($file_language_suffix, $enabled_langcodes)) {
      $errors[] = $this->t('The file name should specify a valid language code as the suffix of the file name separated by a hyphen or underscore. Example: myfile-en.pdf.', [
        '%filename' => $filename,
      ]);
      $errors[] = $this->t('Only the following languages are allowed: %languages-allowed.', [
        '%languages-allowed' => implode(', ', $enabled_langcodes),
      ]);
    }

    if ($current_language !== $default_language) {

      if ($file_language_suffix !== $current_language) {
        $errors[] = $this->t('The file name suffix must match the current page language (%current_language). The provided suffix is %provided_suffix.', [
          '%current_language' => $current_language,
          '%provided_suffix' => $file_language_suffix,
        ]);
      }
    }

    return $errors;
  }

  /**
   * Get prefixes from enabled languages.
   */
  public function getLanguageCodes(): array {
    if (!isset($this->languagePrefixes)) {
      $this->languagePrefixes = [];
      $language_prefixes = $this->configFactory->get('language.negotiation')->get('url.prefixes');
      if (!empty($language_prefixes)) {
        foreach ($this->languageManager->getLanguages() as $langCode => $language) {
          $prefix = mb_substr($langCode, 0, 2);
          $this->languagePrefixes[$langCode] = $prefix;
        }
      }
    }
    return $this->languagePrefixes;
  }

}
