<?php

declare(strict_types=1);

namespace Drupal\field_file_and_translations\Plugin\Validation\Constraint;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Drupal\file\FileInterface;

/**
 * ConstraintValidator to avoid files with the same language.
 */
class UniqueLanguageSuffixConstraintValidator extends ConstraintValidator implements ContainerInjectionInterface {

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a new Multiupload Translation Handler.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface
   *   The language manager.
   */
  public function __construct(LanguageManagerInterface $languageManager) {
    $this->languageManager = $languageManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('language_manager')
    );

  }

  /**
   * {@inheritdoc}
   */
  public function validate($value, Constraint $constraint) {
    $suffixes = [];

    foreach ($value as $item) {

      /** @var \Drupal\file\FileInterface $file */
      $file = $item->entity;
      if ($file instanceof FileInterface) {
        $language_suffix = $this->getLanguageSuffix($file->getFilename());

        if (in_array($language_suffix, $suffixes)) {
          $this->context->addViolation($constraint->notUnique, ['%suffix' => $language_suffix]);
          return;
        }
        $suffixes[] = $language_suffix;
      }
    }
  }

  /**
   * Get the language suffix.
   */
  private function getLanguageSuffix(string $filename): string {
    if (preg_match('/[_-]([a-zA-Z]{2})(_\d+)?\.[a-zA-Z]+$/i', str_replace(' ', '', $filename), $match)) {
      return mb_strtolower($match[1]);
    }
    return $this->languageManager->getDefaultLanguage()->getId();
  }

}
