<?php

declare(strict_types=1);

namespace Drupal\Tests\field_file_and_translations\Functional;

use Drupal\document_multiupload\DocumentMultiuploadMediaFormHandler;
use Drupal\language\Entity\ConfigurableLanguage;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\language\Traits\LanguageTestTrait;

/**
 * Provides tests methods for document media bundle.
 */
class FileAndTranslationMediaTest extends BrowserTestBase {

  use LanguageTestTrait;

  /**
   * Modules to enable.
   *
   * @var array
   */
  protected static $modules = [
    'node',
    'system',
    'field_file_and_translations',
    'file',
    'file_link',
    'link',
    'options',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->drupalLogin($this->drupalCreateUser([], '', TRUE));
    $this->enableTestingLanguages();
  }

  /**
   * Tests all filenames must have language suffix.
   */
  public function testLanguageSuffix(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->fillField('Name', 'Document 1');
  //  $this->assertSession()->pageTextContains(DocumentMultiuploadMediaFormHandler::MULTIPLE_FILE_VALIDATION_MESSAGE);
    $this->attachFileToField('sample.pdf');
    $this->submitForm([], 'Upload');

    $this->assertSession()->pageTextContains('The file name should specify the language code as the suffix of the file name separated by a hyphen or underscore. Example: myfile-en.pdf.');
    $this->assertSession()->elementNotExists('css', '#edit-oe-media-file-table');
  }

  /**
   * Tests English language is required.
   */
  public function testEnglishLanguage(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->fillField('Name', 'Document 1');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');

    $this->attachFileToField('sample_es.pdf');
    $this->submitForm([], 'Upload');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('should specify the English language code as the suffix of the file name separated by a hyphen or underscore. Remove all and upload valid files.');

    $this->submitForm([], 'Remove all');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('Document Document 1 has been created.');
  }

  /**
   * Test English required when multiple files uploaded.
   */
  public function testEnglishIsRequired(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');
    $this->getSession()->getPage()->fillField('Name', 'Document 2');

    $this->attachFileToField('sample_es.pdf');
    $this->submitForm([], 'Upload');

    $this->assertSession()->pageTextContains('should specify the English language code as the suffix of the file name separated by a hyphen or underscore. Remove all and upload valid files.');

    $this->submitForm([], 'Remove all');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_es.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_pt.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_it.pdf');
    $this->submitForm([], 'Upload');

    $this->assertSession()->pageTextMatches('/sample_it(_\d+)?\.pdf.*2\.96 KB/');
    $this->assertSession()->pageTextMatches('/sample_es(_\d+)?\.pdf.*2\.96 KB/');
    $this->assertSession()->pageTextMatches('/sample_pt(_\d+)?\.pdf.*2\.96 KB/');
    $this->assertSession()->pageTextMatches('/sample_en(_\d+)?\.pdf.*2\.96 KB/');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('Document Document 2 has been created.');
  }

  /**
   * Test only can be uploaded files from enabled languages.
   */
  public function testLanguagesEnabled(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');
    $this->getSession()->getPage()->fillField('Name', 'Document 3');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_xx.pdf');
    $this->submitForm([], 'Upload');

    $this->assertSession()->pageTextContains('Only the following languages are allowed: en, it, pt, es.');
  }

  /**
   * Test editing a non-english document translation, same suffix is required.
   */
  public function testEditTranslation(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');
    $this->getSession()->getPage()->fillField('Name', 'Document 3');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_es.pdf');
    $this->submitForm([], 'Upload');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('Document Document 3 has been created.');

    $this->drupalGet('media/1/edit');

    $this->assertSession()->pageTextContains(DocumentMultiuploadMediaFormHandler::MULTIPLE_FILE_VALIDATION_MESSAGE);

    $this->drupalGet('es/media/1/edit');

    $this->assertSession()->pageTextNotContains('Upload translation file. Only filenames with es language suffix is allowed.');
  }

  /**
   * Test only can upload multiple files when editing english document.
   */
  public function testEditEnglishMultipleFiles(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');
    $this->getSession()->getPage()->fillField('Name', 'Document 3');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample_es.pdf');
    $this->submitForm([], 'Upload');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('Document Document 3 has been created.');

    $this->drupalGet('media/1/edit');

    $this->assertSession()->pageTextContains(DocumentMultiuploadMediaFormHandler::MULTIPLE_FILE_VALIDATION_MESSAGE);

    $this->drupalGet('es/media/1/edit');

    $this->assertSession()->pageTextNotContains('Upload translation file. Only filenames with es language suffix is allowed.');
  }

  /**
   * Test can't upload twice files with the same suffix.
   */
  public function testUploadSuffixTwice(): void {
    $this->drupalGet('media/add/document');
    $this->getSession()->getPage()->selectFieldOption('File Type', 'Local');
    $this->getSession()->getPage()->fillField('Name', 'Document 4');

    $this->attachFileToField('sample_en.pdf');
    $this->submitForm([], 'Upload');

    $this->attachFileToField('sample2_en.pdf');
    $this->submitForm([], 'Upload');

    $this->submitForm([], 'Save');

    $this->assertSession()->pageTextContains('More than one file language extension detected. Please, remove all and add only one file per language.');
  }




}
